@@ -23,6 +23,12 @@ const (
 	dateFormat     = "2006-01-02"
 )
 
+const (
+	RSMappingEnvVar = "PBM_REPLSET_REMAPPING"
+	RSMappingFlag   = "replset-remapping"
+	RSMappingDoc    = "re-map replset names for backups/oplog (e.g. to_name_1=from_name_1,to_name_2=from_name_2)"
+)
+
 type outFormat string
 
 const (
@@ -67,7 +73,6 @@ func Main() {
 	backupCmd := pbmCmd.Command("backup", "Make backup")
 	backup := backupOpts{}
 	backupCmd.Flag("compression", "Compression type <none>/<gzip>/<snappy>/<lz4>/<s2>/<pgzip>/<zstd>").
-		Default(string(pbm.CompressionTypeS2)).
 		EnumVar(&backup.compression,
 			string(pbm.CompressionTypeNone), string(pbm.CompressionTypeGZIP),
 			string(pbm.CompressionTypeSNAPPY), string(pbm.CompressionTypeLZ4),
@@ -82,6 +87,7 @@ func Main() {
 		)
 	backupCmd.Flag("compression-level", "Compression level (specific to the compression type)").
 		IntsVar(&backup.compressionLevel)
+	backupCmd.Flag("wait", "Wait for the backup to finish").Short('w').BoolVar(&backup.wait)
 
 	cancelBcpCmd := pbmCmd.Command("cancel-backup", "Cancel backup")
 
@@ -91,19 +97,23 @@ func Main() {
 	restoreCmd.Flag("time", fmt.Sprintf("Restore to the point-in-time. Set in format %s", datetimeFormat)).StringVar(&restore.pitr)
 	restoreCmd.Flag("base-snapshot", "Override setting: Name of older snapshot that PITR will be based on during restore.").StringVar(&restore.pitrBase)
 	restoreCmd.Flag("wait", "Wait for the restore to finish.").Short('w').BoolVar(&restore.wait)
+	restoreCmd.Flag(RSMappingFlag, RSMappingDoc).Envar(RSMappingEnvVar).StringVar(&restore.rsMap)
 
 	replayCmd := pbmCmd.Command("oplog-replay", "Replay oplog")
 	replayOpts := replayOptions{}
 	replayCmd.Flag("start", fmt.Sprintf("Replay oplog from the time. Set in format %s", datetimeFormat)).Required().StringVar(&replayOpts.start)
 	replayCmd.Flag("end", "Replay oplog to the time. Set in format %s").Required().StringVar(&replayOpts.end)
 	replayCmd.Flag("wait", "Wait for the restore to finish.").Short('w').BoolVar(&replayOpts.wait)
+	replayCmd.Flag(RSMappingFlag, RSMappingDoc).Envar(RSMappingEnvVar).StringVar(&replayOpts.rsMap)
+	// todo(add oplog cancel)
 
 	listCmd := pbmCmd.Command("list", "Backup list")
 	list := listOpts{}
 	listCmd.Flag("restore", "Show last N restores").Default("false").BoolVar(&list.restore)
-	listCmd.Flag("oplog-replay", "Show last N oplog replays").Default("false").BoolVar(&list.oplogReplay)
+	listCmd.Flag("unbacked", "Show unbacked oplog ranges").Default("false").BoolVar(&list.unbacked)
 	listCmd.Flag("full", "Show extended restore info").Default("false").Short('f').Hidden().BoolVar(&list.full)
 	listCmd.Flag("size", "Show last N backups").Default("0").IntVar(&list.size)
+	listCmd.Flag(RSMappingFlag, RSMappingDoc).Envar(RSMappingEnvVar).StringVar(&list.rsMap)
 
 	deleteBcpCmd := pbmCmd.Command("delete-backup", "Delete a backup")
 	deleteBcp := deleteBcpOpts{}
@@ -126,8 +136,11 @@ func Main() {
 	logsCmd.Flag("opid", "Operation ID").Short('i').StringVar(&logs.opid)
 	logsCmd.Flag("extra", "Show extra data in text format").Hidden().Short('x').BoolVar(&logs.extr)
 
+	statusOpts := statusOptions{}
 	statusCmd := pbmCmd.Command("status", "Show PBM status")
-	statusSection := statusCmd.Flag("sections", "Sections of status to display <cluster>/<pitr>/<running>/<backups>.").Short('s').Enums("cluster", "pitr", "running", "backups")
+	statusCmd.Flag(RSMappingFlag, RSMappingDoc).Envar(RSMappingEnvVar).StringVar(&statusOpts.rsMap)
+	statusCmd.Flag("sections", "Sections of status to display <cluster>/<pitr>/<running>/<backups>.").Short('s').
+		EnumsVar(&statusOpts.sections, "cluster", "pitr", "running", "backups")
 
 	cmd, err := pbmCmd.DefaultEnvars().Parse(os.Args[1:])
 	if err != nil {
@@ -164,6 +177,7 @@ func Main() {
 	if err != nil {
 		exitErr(errors.Wrap(err, "connect to mongodb"), pbmOutF)
 	}
+
 	pbmClient.InitLogger("", "")
 
 	switch cmd {
@@ -187,7 +201,7 @@ func Main() {
 	case logsCmd.FullCommand():
 		out, err = runLogs(pbmClient, &logs)
 	case statusCmd.FullCommand():
-		out, err = status(pbmClient, *mURL, statusSection, pbmOutF == outJSONpretty)
+		out, err = status(pbmClient, *mURL, statusOpts, pbmOutF == outJSONpretty)
 	}
 
 	if err != nil {
@@ -308,19 +322,24 @@ type snapshotStat struct {
 	Name       string         `json:"name"`
 	Size       int64          `json:"size,omitempty"`
 	Status     pbm.Status     `json:"status"`
-	Err        string         `json:"error,omitempty"`
+	Err        error          `json:"error,omitempty"`
 	StateTS    int64          `json:"completeTS"`
 	PBMVersion string         `json:"pbmVersion"`
 	Type       pbm.BackupType `json:"type"`
 }
 
 type pitrRange struct {
-	Err   string       `json:"error,omitempty"`
-	Range pbm.Timeline `json:"range"`
+	Err            error        `json:"error,omitempty"`
+	Range          pbm.Timeline `json:"range"`
+	NoBaseSnapshot bool         `json:"noBaseSnapshot,omitempty"`
+}
+
+func (pr pitrRange) String() string {
+	return fmt.Sprintf("{ %s }", pr.Range)
 }
 
 func fmtTS(ts int64) string {
-	return time.Unix(ts, 0).UTC().Format("2006-01-02T15:04:05")
+	return time.Unix(ts, 0).UTC().Format(time.RFC3339)
 }
 
 type outMsg struct {
