@@ -3,19 +3,21 @@ package cli
 import (
 	"encoding/json"
 	"fmt"
+	"sort"
 	"time"
 
-	"github.com/percona/percona-backup-mongodb/pbm"
-	"github.com/percona/percona-backup-mongodb/version"
 	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson/primitive"
+
+	"github.com/percona/percona-backup-mongodb/pbm"
 )
 
 type listOpts struct {
-	restore     bool
-	oplogReplay bool
-	full        bool
-	size        int
+	restore  bool
+	unbacked bool
+	full     bool
+	size     int
+	rsMap    string
 }
 
 type restoreStatus struct {
@@ -78,16 +80,21 @@ func (r restoreListOut) MarshalJSON() ([]byte, error) {
 }
 
 func runList(cn *pbm.PBM, l *listOpts) (fmt.Stringer, error) {
+	rsMap, err := parseRSNamesMapping(l.rsMap)
+	if err != nil {
+		return nil, errors.WithMessage(err, "cannot parse replset mapping")
+	}
+
 	if l.restore {
 		return restoreList(cn, int64(l.size), l.full)
 	}
-	// show message ans skip when resync is running
+	// show message and skip when resync is running
 	lk, err := findLock(cn, cn.GetLocks)
 	if err == nil && lk != nil && lk.Type == pbm.CmdResync {
 		return outMsg{"Storage resync is running. Backups list will be available after sync finishes."}, nil
 	}
 
-	return backupList(cn, l.size, l.full)
+	return backupList(cn, l.size, l.full, l.unbacked, rsMap)
 }
 
 func restoreList(cn *pbm.PBM, size int64, full bool) (*restoreListOut, error) {
@@ -136,6 +143,10 @@ type backupListOut struct {
 
 func (bl backupListOut) String() string {
 	s := fmt.Sprintln("Backup snapshots:")
+
+	sort.Slice(bl.Snapshots, func(i, j int) bool {
+		return bl.Snapshots[i].StateTS < bl.Snapshots[j].StateTS
+	})
 	for _, b := range bl.Snapshots {
 		s += fmt.Sprintf("  %s <%s> [complete: %s]\n", b.Name, b.Type, fmtTS(int64(b.StateTS)))
 	}
@@ -145,8 +156,15 @@ func (bl backupListOut) String() string {
 		s += fmt.Sprintln("\nPITR <off>:")
 	}
 
+	sort.Slice(bl.PITR.Ranges, func(i, j int) bool {
+		return bl.PITR.Ranges[i].Range.End < bl.PITR.Ranges[j].Range.End
+	})
 	for _, r := range bl.PITR.Ranges {
-		s += fmt.Sprintf("  %s - %s\n", fmtTS(int64(r.Range.Start)), fmtTS(int64(r.Range.End)))
+		f := ""
+		if r.NoBaseSnapshot {
+			f = " (no base snapshot)"
+		}
+		s += fmt.Sprintf("  %s - %s%s\n", fmtTS(int64(r.Range.Start)), fmtTS(int64(r.Range.End)), f)
 	}
 	if bl.PITR.RsRanges != nil {
 		s += "\n"
@@ -158,12 +176,12 @@ func (bl backupListOut) String() string {
 	return s
 }
 
-func backupList(cn *pbm.PBM, size int, full bool) (list backupListOut, err error) {
-	list.Snapshots, err = getSnapshotList(cn, size)
+func backupList(cn *pbm.PBM, size int, full, unbacked bool, rsMap map[string]string) (list backupListOut, err error) {
+	list.Snapshots, err = getSnapshotList(cn, size, rsMap)
 	if err != nil {
 		return list, errors.Wrap(err, "get snapshots")
 	}
-	list.PITR.Ranges, list.PITR.RsRanges, err = getPitrList(cn, size, full)
+	list.PITR.Ranges, list.PITR.RsRanges, err = getPitrList(cn, size, full, unbacked, rsMap)
 	if err != nil {
 		return list, errors.Wrap(err, "get PITR ranges")
 	}
@@ -176,7 +194,7 @@ func backupList(cn *pbm.PBM, size int, full bool) (list backupListOut, err error
 	return list, nil
 }
 
-func getSnapshotList(cn *pbm.PBM, size int) (s []snapshotStat, err error) {
+func getSnapshotList(cn *pbm.PBM, size int, rsMap map[string]string) (s []snapshotStat, err error) {
 	bcps, err := cn.BackupsList(int64(size))
 	if err != nil {
 		return nil, errors.Wrap(err, "unable to get backups list")
@@ -194,7 +212,7 @@ func getSnapshotList(cn *pbm.PBM, size int) (s []snapshotStat, err error) {
 
 	// pbm.PBM is always connected either to config server or to the sole (hence main) RS
 	// which the `confsrv` param in `bcpMatchCluster` is all about
-	bcpsMatchCluster(bcps, shards, inf.SetName)
+	bcpsMatchCluster(bcps, shards, inf.SetName, rsMap)
 
 	for i := len(bcps) - 1; i >= 0; i-- {
 		b := bcps[i]
@@ -202,9 +220,6 @@ func getSnapshotList(cn *pbm.PBM, size int) (s []snapshotStat, err error) {
 		if b.Status != pbm.StatusDone {
 			continue
 		}
-		if !version.Compatible(version.DefaultInfo.Version, b.PBMVersion) {
-			continue
-		}
 
 		s = append(s, snapshotStat{
 			Name:       b.Name,
@@ -219,7 +234,7 @@ func getSnapshotList(cn *pbm.PBM, size int) (s []snapshotStat, err error) {
 }
 
 // getPitrList shows only chunks derived from `Done` and compatible version's backups
-func getPitrList(cn *pbm.PBM, size int, full bool) (ranges []pitrRange, rsRanges map[string][]pitrRange, err error) {
+func getPitrList(cn *pbm.PBM, size int, full, unbacked bool, rsMap map[string]string) (ranges []pitrRange, rsRanges map[string][]pitrRange, err error) {
 	inf, err := cn.GetNodeInfo()
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "define cluster state")
@@ -235,10 +250,11 @@ func getPitrList(cn *pbm.PBM, size int, full bool) (ranges []pitrRange, rsRanges
 		return nil, nil, errors.Wrap(err, "get cluster time")
 	}
 
+	mapRevRS := pbm.MakeReverseRSMapFunc(rsMap)
 	rsRanges = make(map[string][]pitrRange)
 	var rstlines [][]pbm.Timeline
 	for _, s := range shards {
-		tlns, err := cn.PITRGetValidTimelines(s.RS, now, nil)
+		tlns, err := cn.PITRGetValidTimelines(mapRevRS(s.RS), now, nil)
 		if err != nil {
 			return nil, nil, errors.Wrapf(err, "get PITR timelines for %s replset", s.RS)
 		}
@@ -261,29 +277,76 @@ func getPitrList(cn *pbm.PBM, size int, full bool) (ranges []pitrRange, rsRanges
 		rstlines = append(rstlines, tlns)
 	}
 
-	sh := make(map[string]struct{}, len(shards))
+	sh := make(map[string]bool, len(shards))
 	for _, s := range shards {
-		sh[s.RS] = struct{}{}
+		sh[s.RS] = s.RS == inf.SetName
 	}
 
-	var buf []string
 	for _, tl := range pbm.MergeTimelines(rstlines...) {
-		bcp, err := cn.GetLastBackup(&primitive.Timestamp{T: tl.End, I: 0})
-		if errors.Is(err, pbm.ErrNotFound) {
-			continue
-		}
+		lastWrite, err := getBaseSnapshotLastWrite(cn, sh, rsMap, tl)
 		if err != nil {
-			return nil, nil, errors.Wrapf(err, "get backup for timeline: %s", tl)
+			return nil, nil, err
 		}
-		buf = buf[:0]
-		bcpMatchCluster(bcp, sh, inf.SetName, &buf)
 
-		if bcp.Status != pbm.StatusDone || !version.Compatible(version.DefaultInfo.Version, bcp.PBMVersion) {
-			continue
+		rs := splitByBaseSnapshot(lastWrite, tl)
+		for i := range rs {
+			if !unbacked && rs[i].NoBaseSnapshot {
+				continue
+			}
+
+			ranges = append(ranges, rs[i])
 		}
-		tl.Start++
-		ranges = append(ranges, pitrRange{Range: tl})
 	}
 
 	return ranges, rsRanges, nil
 }
+
+func getBaseSnapshotLastWrite(cn *pbm.PBM, sh map[string]bool, rsMap map[string]string, tl pbm.Timeline) (*primitive.Timestamp, error) {
+	bcp, err := cn.GetFirstBackup(&primitive.Timestamp{T: tl.Start, I: 0})
+	if err != nil {
+		if !errors.Is(err, pbm.ErrNotFound) {
+			return nil, errors.Wrapf(err, "get backup for timeline: %s", tl)
+		}
+
+		return nil, nil
+	}
+	if bcp == nil {
+		return nil, nil
+	}
+
+	bcpMatchCluster(bcp, sh, pbm.MakeRSMapFunc(rsMap), pbm.MakeReverseRSMapFunc(rsMap))
+
+	if bcp.Status != pbm.StatusDone {
+		return nil, nil
+	}
+
+	return &bcp.LastWriteTS, nil
+}
+
+func splitByBaseSnapshot(lastWrite *primitive.Timestamp, tl pbm.Timeline) []pitrRange {
+	if lastWrite == nil || (lastWrite.T < tl.Start || lastWrite.T > tl.End) {
+		return []pitrRange{{Range: tl, NoBaseSnapshot: true}}
+	}
+
+	ranges := make([]pitrRange, 0, 1)
+
+	if lastWrite.T <= tl.End {
+		ranges = append(ranges, pitrRange{
+			NoBaseSnapshot: true,
+			Range: pbm.Timeline{
+				Start: tl.Start,
+				End:   lastWrite.T,
+			},
+		})
+	}
+	if lastWrite.T >= tl.Start {
+		ranges = append(ranges, pitrRange{
+			Range: pbm.Timeline{
+				Start: lastWrite.T + 1,
+				End:   tl.End,
+			},
+		})
+	}
+
+	return ranges
+}
