@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"log"
 	"net/url"
+	"sort"
 	"strings"
 	"time"
 
@@ -21,6 +22,11 @@ import (
 	"github.com/percona/percona-backup-mongodb/version"
 )
 
+type statusOptions struct {
+	rsMap    string
+	sections []string
+}
+
 type statusOut struct {
 	data   []*statusSect
 	pretty bool
@@ -77,30 +83,40 @@ func (s statusOut) set(cn *pbm.PBM, curi string, sfilter map[string]bool) (err e
 	return nil
 }
 
-func status(cn *pbm.PBM, curi string, showSection *[]string, pretty bool) (fmt.Stringer, error) {
+func status(cn *pbm.PBM, curi string, opts statusOptions, pretty bool) (fmt.Stringer, error) {
+	rsMap, err := parseRSNamesMapping(opts.rsMap)
+	if err != nil {
+		return nil, errors.WithMessage(err, "cannot parse replset mapping")
+	}
+
+	storageStatFn := func(cn *pbm.PBM) (fmt.Stringer, error) {
+		return getStorageStat(cn, rsMap)
+	}
+
 	out := statusOut{
 		data: []*statusSect{
-			{"cluster", "Cluster", nil,
+			{
+				"cluster", "Cluster", nil,
 				func(cn *pbm.PBM) (fmt.Stringer, error) {
 					return clusterStatus(cn, curi)
 				},
 			},
 			{"pitr", "PITR incremental backup", nil, getPitrStatus},
 			{"running", "Currently running", nil, getCurrOps},
-			{"backups", "Backups", nil, getStorageStat},
+			{"backups", "Backups", nil, storageStatFn},
 		},
 		pretty: pretty,
 	}
 
 	var sfilter map[string]bool
-	if showSection != nil && len(*showSection) > 0 {
+	if opts.sections != nil && len(opts.sections) > 0 {
 		sfilter = make(map[string]bool)
-		for _, s := range *showSection {
+		for _, s := range opts.sections {
 			sfilter[s] = true
 		}
 	}
 
-	err := out.set(cn, curi, sfilter)
+	err = out.set(cn, curi, sfilter)
 
 	return out, err
 }
@@ -457,29 +473,35 @@ type pitrRanges struct {
 
 func (s storageStat) String() string {
 	ret := fmt.Sprintf("%s %s %s\n", s.Type, s.Region, s.Path)
-	if len(s.Snapshot) == 0 {
+	if len(s.Snapshot) == 0 && len(s.PITR.Ranges) == 0 {
 		return ret + "  (none)"
 	}
 
 	ret += fmt.Sprintln("  Snapshots:")
+
+	sort.Slice(s.Snapshot, func(i, j int) bool {
+		a, b := s.Snapshot[i], s.Snapshot[j]
+		return a.StateTS > b.StateTS
+	})
+
 	for _, sn := range s.Snapshot {
 		var status string
 		switch sn.Status {
 		case pbm.StatusDone:
-			status = fmt.Sprintf(" <%s> [complete: %s]", sn.Type, fmtTS(sn.StateTS))
+			status = fmt.Sprintf("[complete: %s]", fmtTS(sn.StateTS))
 		case pbm.StatusCancelled:
-			status = fmt.Sprintf(" [!cancelled: %s]", fmtTS(sn.StateTS))
+			status = fmt.Sprintf("[!canceled: %s]", fmtTS(sn.StateTS))
 		case pbm.StatusError:
-			status = fmt.Sprintf(" [ERROR: %s] [%s]", sn.Err, fmtTS(sn.StateTS))
+			if errors.Is(sn.Err, errIncompatible) {
+				status = fmt.Sprintf("[incompatible: %s] [%s]", sn.Err.Error(), fmtTS(sn.StateTS))
+			} else {
+				status = fmt.Sprintf("[ERROR: %s] [%s]", sn.Err.Error(), fmtTS(sn.StateTS))
+			}
 		default:
-			status = fmt.Sprintf(" [running: %s / %s]", sn.Status, fmtTS(sn.StateTS))
+			status = fmt.Sprintf("[running: %s / %s]", sn.Status, fmtTS(sn.StateTS))
 		}
 
-		var v string
-		if !version.Compatible(version.DefaultInfo.Version, sn.PBMVersion) {
-			v = fmt.Sprintf(" !!! backup v%s is not compatible with PBM v%s", sn.PBMVersion, version.DefaultInfo.Version)
-		}
-		ret += fmt.Sprintf("    %s %s%s%s\n", sn.Name, fmtSize(sn.Size), status, v)
+		ret += fmt.Sprintf("    %s %s <%s> %s\n", sn.Name, fmtSize(sn.Size), sn.Type, status)
 	}
 
 	if len(s.PITR.Ranges) == 0 {
@@ -488,18 +510,27 @@ func (s storageStat) String() string {
 
 	ret += fmt.Sprintf("  PITR chunks [%s]:\n", fmtSize(s.PITR.Size))
 
+	sort.Slice(s.PITR.Ranges, func(i, j int) bool {
+		a, b := s.PITR.Ranges[i], s.PITR.Ranges[j]
+		return a.Range.End > b.Range.End
+	})
+
 	for _, sn := range s.PITR.Ranges {
 		var v string
-		if sn.Err != "" {
-			v = fmt.Sprintf(" !!! %s", sn.Err)
+		if sn.Err != nil && !errors.Is(sn.Err, pbm.ErrNotFound) {
+			v = fmt.Sprintf(" !!! %s", sn.Err.Error())
 		}
-		ret += fmt.Sprintf("    %s - %s%s\n", fmtTS(int64(sn.Range.Start)), fmtTS(int64(sn.Range.End)), v)
+		f := ""
+		if sn.NoBaseSnapshot {
+			f = " (no base snapshot)"
+		}
+		ret += fmt.Sprintf("    %s - %s%s%s\n", fmtTS(int64(sn.Range.Start)), fmtTS(int64(sn.Range.End)), f, v)
 	}
 
 	return ret
 }
 
-func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
+func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error) {
 	var s storageStat
 
 	cfg, err := cn.GetConfig()
@@ -531,7 +562,7 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 
 	// pbm.PBM is always connected either to config server or to the sole (hence main) RS
 	// which the `confsrv` param in `bcpMatchCluster` is all about
-	bcpsMatchCluster(bcps, shards, inf.SetName)
+	bcpsMatchCluster(bcps, shards, inf.SetName, rsMap)
 
 	stg, err := cn.GetStorage(cn.Logger().NewEvent("", "", "", primitive.Timestamp{}))
 	if err != nil {
@@ -550,9 +581,15 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 			StateTS:    bcp.LastTransitionTS,
 			PBMVersion: bcp.PBMVersion,
 			Type:       bcp.Type,
+			Err:        bcp.Error(),
 		}
 
 		switch bcp.Status {
+		case pbm.StatusError:
+			if !errors.Is(snpsht.Err, errIncompatible) {
+				break
+			}
+			fallthrough
 		case pbm.StatusDone:
 			snpsht.StateTS = int64(bcp.LastWriteTS.T)
 			var err error
@@ -563,23 +600,21 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 				snpsht.Size, err = getSnapshotSize(bcp.Replsets, stg)
 			}
 			if err != nil {
-				snpsht.Err = err.Error()
+				snpsht.Err = err
 				snpsht.Status = pbm.StatusError
 			}
-		case pbm.StatusError:
-			snpsht.Err = bcp.Error
 		case pbm.StatusCancelled:
 			// leave as it is, not to rewrite status with the `stuck` error
 		default:
 			if bcp.Hb.T+pbm.StaleFrameSec < now.T {
-				snpsht.Err = fmt.Sprintf("Backup stuck at `%v` stage, last beat ts: %d", bcp.Status, bcp.Hb.T)
+				snpsht.Err = fmt.Errorf("Backup stuck at `%v` stage, last beat ts: %d", bcp.Status, bcp.Hb.T)
 				snpsht.Status = pbm.StatusError
 			}
 		}
 		s.Snapshot = append(s.Snapshot, snpsht)
 	}
 
-	s.PITR, err = getPITRranges(cn, stg)
+	s.PITR, err = getPITRranges(cn, stg, bcps, rsMap)
 	if err != nil {
 		return s, errors.Wrap(err, "get PITR chunks")
 	}
@@ -587,7 +622,7 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 	return s, nil
 }
 
-func getPITRranges(cn *pbm.PBM, stg storage.Storage) (*pitrRanges, error) {
+func getPITRranges(cn *pbm.PBM, stg storage.Storage, bcps []pbm.BackupMeta, rsMap map[string]string) (*pitrRanges, error) {
 	shards, err := cn.ClusterMembers()
 	if err != nil {
 		return nil, errors.Wrap(err, "get cluster members")
@@ -608,13 +643,13 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage) (*pitrRanges, error) {
 		return nil, errors.Wrap(err, "get cluster time")
 	}
 
+	mapRevRS := pbm.MakeReverseRSMapFunc(rsMap)
 	var size int64
 	var rstlines [][]pbm.Timeline
 	for _, s := range shards {
-		tlns, err := cn.PITRGetValidTimelines(s.RS, now, flist)
+		tlns, err := cn.PITRGetValidTimelines(mapRevRS(s.RS), now, flist)
 		if err != nil {
-			log.Printf("ERROR: get PITR timelines for %s replset: %v", s.RS, err)
-			continue
+			return nil, errors.Wrapf(err, "get PITR timelines for %s replset: %s", s.RS, err)
 		}
 		if tlns == nil {
 			continue
@@ -627,26 +662,32 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage) (*pitrRanges, error) {
 		}
 	}
 
-	merged := pbm.MergeTimelines(rstlines...)
+	sort.Slice(bcps, func(i, j int) bool {
+		return primitive.CompareTimestamp(bcps[i].LastWriteTS, bcps[j].LastWriteTS) == -1
+	})
 
 	var pr []pitrRange
-	for i := len(merged) - 1; i >= 0; i-- {
-		tl := merged[i]
-		var rng pitrRange
-		rng.Range.Start = tl.Start + 1
-		rng.Range.End = tl.End
+	for _, tl := range pbm.MergeTimelines(rstlines...) {
+		var bcplastWrite *primitive.Timestamp
 
-		bcp, err := cn.GetLastBackup(&primitive.Timestamp{T: tl.End, I: 0})
-		if err != nil && errors.Is(err, pbm.ErrNotFound) {
-			log.Printf("ERROR: get backup for timeline: %s", tl)
-			continue
+		for _, bcp := range bcps {
+			if bcp.Error() != nil {
+				continue
+			}
+			if !version.Compatible(version.DefaultInfo.Version, bcp.PBMVersion) {
+				bcp.SetRuntimeError(errIncompatibleVersion{bcp.PBMVersion})
+				continue
+			}
+
+			if bcp.LastWriteTS.T < tl.Start && bcp.FirstWriteTS.T > tl.End {
+				continue
+			}
+
+			bcplastWrite = &bcp.LastWriteTS
+			break
 		}
-		if errors.Is(err, pbm.ErrNotFound) {
-			rng.Err = "no backup found"
-		} else if !version.Compatible(version.DefaultInfo.Version, bcp.PBMVersion) {
-			rng.Err = fmt.Sprintf("backup v%s is not compatible with PBM v%s", bcp.PBMVersion, version.DefaultInfo.Version)
-		}
-		pr = append(pr, rng)
+
+		pr = append(pr, splitByBaseSnapshot(bcplastWrite, tl)...)
 	}
 
 	return &pitrRanges{Ranges: pr, Size: size}, nil
