@@ -36,8 +36,9 @@ func (c *Cluster) RestartAgents() {
 		log.Fatalf("ERROR: get metadata for the backup %s: %v", bcpName, err)
 	}
 
-	if meta.Status != pbm.StatusError && meta.Error != pbmLostAgentsErr {
-		log.Fatalf("ERROR: wrong state of the backup %s. Expect: %s/%s. Got: %s/%s", bcpName, pbm.StatusError, pbmLostAgentsErr, meta.Status, meta.Error)
+	if meta.Status != pbm.StatusError || meta.Error() == nil || meta.Error().Error() != pbmLostAgentsErr {
+		log.Fatalf("ERROR: wrong state of the backup %s. Expect: %s/%s. Got: %s/%s",
+			bcpName, pbm.StatusError, pbmLostAgentsErr, meta.Status, meta.Error())
 	}
 
 	for rs := range c.shards {
