@@ -37,10 +37,11 @@ func (c *Cluster) NetworkCut() {
 		log.Fatalf("ERROR: get metadata for the backup %s: %v", bcpName, err)
 	}
 
-	if meta.Status != pbm.StatusError && meta.Error != pbmLostAgentsErr {
-		log.Fatalf("ERROR: wrong state of the backup %s. Expect: %s/%s. Got: %s/%s", bcpName, pbm.StatusError, pbmLostAgentsErr, meta.Status, meta.Error)
+	if meta.Status != pbm.StatusError || meta.Error() == nil || meta.Error().Error() != pbmLostAgentsErr {
+		log.Fatalf("ERROR: wrong state of the backup %s. Expect: %s/%s. Got: %s/%s",
+			bcpName, pbm.StatusError, pbmLostAgentsErr, meta.Status, meta.Error())
 	}
-	log.Printf("Backup status %s/%s\n", meta.Status, meta.Error)
+	log.Printf("Backup status %s/%s\n", meta.Status, meta.Error())
 
 	log.Println("Restore network on agents", rs)
 	err = c.docker.RunOnReplSet(rs, time.Second*5,
