@@ -1,6 +1,7 @@
 package pbm
 
 import (
+	"fmt"
 	"time"
 
 	"github.com/pkg/errors"
@@ -11,27 +12,81 @@ import (
 )
 
 type RestoreMeta struct {
-	OPID             string              `bson:"opid" json:"opid"`
+	Status           Status              `bson:"status" json:"status"`
+	Error            string              `bson:"error,omitempty" json:"error,omitempty"`
 	Name             string              `bson:"name" json:"name"`
+	OPID             string              `bson:"opid" json:"opid"`
 	Backup           string              `bson:"backup" json:"backup"`
+	StartPITR        int64               `bson:"start_pitr" json:"start_pitr"`
 	PITR             int64               `bson:"pitr" json:"pitr"`
 	Replsets         []RestoreReplset    `bson:"replsets" json:"replsets"`
 	Hb               primitive.Timestamp `bson:"hb" json:"hb"`
 	StartTS          int64               `bson:"start_ts" json:"start_ts"`
 	LastTransitionTS int64               `bson:"last_transition_ts" json:"last_transition_ts"`
-	Status           Status              `bson:"status" json:"status"`
 	Conditions       []Condition         `bson:"conditions" json:"conditions"`
-	Error            string              `bson:"error,omitempty" json:"error,omitempty"`
+	Type             BackupType          `bson:"type" json:"type"`
+	Leader           string              `bson:"l,omitempty" json:"l,omitempty"`
 }
 
 type RestoreReplset struct {
 	Name             string              `bson:"name" json:"name"`
 	StartTS          int64               `bson:"start_ts" json:"start_ts"`
 	Status           Status              `bson:"status" json:"status"`
+	Txn              RestoreTxn          `bson:"txn" json:"txn"`
+	CurrentOp        primitive.Timestamp `bson:"op" json:"op"`
 	LastTransitionTS int64               `bson:"last_transition_ts" json:"last_transition_ts"`
 	LastWriteTS      primitive.Timestamp `bson:"last_write_ts" json:"last_write_ts"`
+	Nodes            []RestoreNode       `bson:"nodes,omitempty" json:"nodes,omitempty"`
 	Error            string              `bson:"error,omitempty" json:"error,omitempty"`
 	Conditions       []Condition         `bson:"conditions" json:"conditions"`
+	Hb               primitive.Timestamp `bson:"hb" json:"hb"`
+}
+
+type RestoreNode struct {
+	Name             string      `bson:"name" json:"name"`
+	Status           Status      `bson:"status" json:"status"`
+	LastTransitionTS int64       `bson:"last_transition_ts" json:"last_transition_ts"`
+	Error            string      `bson:"error,omitempty" json:"error,omitempty"`
+	Conditions       []Condition `bson:"conditions" json:"conditions"`
+}
+
+type TxnState string
+
+const (
+	TxnCommit  TxnState = "commit"
+	TxnPrepare TxnState = "prepare"
+	TxnAbort   TxnState = "abort"
+	TxnUnknown TxnState = ""
+)
+
+type RestoreTxn struct {
+	ID    string              `bson:"id" json:"id"`
+	Ctime primitive.Timestamp `bson:"ts" json:"ts"` // commit timestamp of the transaction
+	State TxnState            `bson:"state" json:"state"`
+}
+
+func (t RestoreTxn) String() string {
+	return fmt.Sprintf("<%s> [%s] %v", t.ID, t.State, t.Ctime)
+}
+
+func (p *PBM) RestoreSetRSTxn(name string, rsName string, txn RestoreTxn) error {
+	_, err := p.Conn.Database(DB).Collection(RestoresCollection).UpdateOne(
+		p.ctx,
+		bson.D{{"name", name}, {"replsets.name", rsName}},
+		bson.D{{"$set", bson.M{"replsets.$.txn": txn}}},
+	)
+
+	return err
+}
+
+func (p *PBM) SetCurrentOp(name string, rsName string, ts primitive.Timestamp) error {
+	_, err := p.Conn.Database(DB).Collection(RestoresCollection).UpdateOne(
+		p.ctx,
+		bson.D{{"name", name}, {"replsets.name", rsName}},
+		bson.D{{"$set", bson.M{"replsets.$.op": ts}}},
+	)
+
+	return err
 }
 
 func (p *PBM) SetRestoreMeta(m *RestoreMeta) error {
@@ -155,13 +210,11 @@ func (p *PBM) SetRestoreBackup(name, backupName string) error {
 	return err
 }
 
-func (p *PBM) SetRestorePITR(name string, ts int64) error {
+func (p *PBM) SetOplogTimestamps(name string, start, end int64) error {
 	_, err := p.Conn.Database(DB).Collection(RestoresCollection).UpdateOne(
 		p.ctx,
-		bson.D{{"name", name}},
-		bson.D{
-			{"$set", bson.M{"pitr": ts}},
-		},
+		bson.M{"name": name},
+		bson.M{"$set": bson.M{"start_pitr": start, "pitr": end}},
 	)
 
 	return err
