@@ -2,6 +2,7 @@ package restore
 
 import (
 	"context"
+	"fmt"
 	"io"
 	"strings"
 	"time"
@@ -67,6 +68,7 @@ type Restore struct {
 	//
 	// Only the restore leader would have this info.
 	shards []pbm.Shard
+	rsMap  map[string]string
 
 	oplog *Oplog
 	log   *log.Event
@@ -74,10 +76,15 @@ type Restore struct {
 }
 
 // New creates a new restore object
-func New(cn *pbm.PBM, node *pbm.Node) *Restore {
+func New(cn *pbm.PBM, node *pbm.Node, rsMap map[string]string) *Restore {
+	if rsMap == nil {
+		rsMap = make(map[string]string)
+	}
+
 	return &Restore{
-		cn:   cn,
-		node: node,
+		cn:    cn,
+		node:  node,
+		rsMap: rsMap,
 	}
 }
 
@@ -119,6 +126,16 @@ func (r *Restore) Snapshot(cmd pbm.RestoreCmd, opid pbm.OPID, l *log.Event) (err
 		return err
 	}
 
+	err = r.checkSnapshot(bcp)
+	if err != nil {
+		return err
+	}
+
+	err = r.setShards(bcp)
+	if err != nil {
+		return err
+	}
+
 	dump, oplog, err := r.snapshotObjects(bcp)
 	if err != nil {
 		return err
@@ -150,6 +167,10 @@ func (r *Restore) Snapshot(cmd pbm.RestoreCmd, opid pbm.OPID, l *log.Event) (err
 		return err
 	}
 
+	if err = r.updateRouterConfig(r.cn.Context()); err != nil {
+		return errors.WithMessage(err, "update router config")
+	}
+
 	return r.Done()
 }
 
@@ -194,6 +215,25 @@ func (r *Restore) PITR(cmd pbm.PITRestoreCmd, opid pbm.OPID, l *log.Event) (err
 		return errors.Wrap(err, "set backup name")
 	}
 
+	err = r.checkSnapshot(bcp)
+	if err != nil {
+		return err
+	}
+
+	err = r.setShards(bcp)
+	if err != nil {
+		return err
+	}
+
+	bcpShards := make([]string, len(bcp.Replsets))
+	for i := range bcp.Replsets {
+		bcpShards[i] = bcp.Replsets[i].Name
+	}
+
+	if !sliceHasString(bcpShards, pbm.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)) {
+		return r.Done() // skip. no backup for current rs
+	}
+
 	chunks, err := r.chunks(bcp.LastWriteTS, tsTo)
 	if err != nil {
 		return err
@@ -232,6 +272,10 @@ func (r *Restore) PITR(cmd pbm.PITRestoreCmd, opid pbm.OPID, l *log.Event) (err
 		return err
 	}
 
+	if err = r.updateRouterConfig(r.cn.Context()); err != nil {
+		return errors.WithMessage(err, "update router config")
+	}
+
 	return r.Done()
 }
 
@@ -258,6 +302,20 @@ func (r *Restore) ReplayOplog(cmd pbm.ReplayCmd, opid pbm.OPID, l *log.Event) (e
 		}
 	}
 
+	oplogShards, err := r.cn.AllOplogRSNames(r.cn.Context(), cmd.Start, cmd.End)
+	if err != nil {
+		return err
+	}
+
+	err = r.checkTopologyForOplog(r.cn.Context(), r.shards, oplogShards)
+	if err != nil {
+		return errors.WithMessage(err, "topology")
+	}
+
+	if !sliceHasString(oplogShards, pbm.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)) {
+		return r.Done() // skip. no oplog for current rs
+	}
+
 	chunks, err := r.chunks(cmd.Start, cmd.End)
 	if err != nil {
 		return err
@@ -354,11 +412,37 @@ func (r *Restore) init(name string, opid pbm.OPID, l *log.Event) (err error) {
 	return nil
 }
 
+func (r *Restore) checkTopologyForOplog(ctx context.Context, currShards []pbm.Shard, oplogShards []string) error {
+	mapRS, mapRevRS := pbm.MakeRSMapFunc(r.rsMap), pbm.MakeReverseRSMapFunc(r.rsMap)
+
+	shards := make(map[string]struct{}, len(currShards))
+	for i := range r.shards {
+		shards[r.shards[i].RS] = struct{}{}
+	}
+
+	var missed []string
+	for _, rs := range oplogShards {
+		name := mapRS(rs)
+		if _, ok := shards[name]; !ok {
+			missed = append(missed, name)
+		} else if mapRevRS(name) != rs {
+			missed = append(missed, name)
+		}
+	}
+
+	if len(missed) != 0 {
+		return errors.Errorf("missed replset %s", strings.Join(missed, ", "))
+	}
+
+	return nil
+}
+
 // chunks defines chunks of oplog slice in given range, ensures its integrity (timeline
 // is contiguous - there are no gaps), checks for respective files on storage and returns
 // chunks list if all checks passed
 func (r *Restore) chunks(from, to primitive.Timestamp) ([]pbm.OplogChunk, error) {
-	chunks, err := r.cn.PITRGetChunksSlice(r.nodeInfo.SetName, from, to)
+	mapRevRS := pbm.MakeReverseRSMapFunc(r.rsMap)
+	chunks, err := r.cn.PITRGetChunksSlice(mapRevRS(r.nodeInfo.SetName), from, to)
 	if err != nil {
 		return nil, errors.Wrap(err, "get chunks index")
 	}
@@ -413,11 +497,17 @@ func (r *Restore) setShards(bcp *pbm.BackupMeta) error {
 		fl[rs.RS] = rs
 	}
 
+	mapRS, mapRevRS := pbm.MakeRSMapFunc(r.rsMap), pbm.MakeReverseRSMapFunc(r.rsMap)
+
 	var nors []string
 	for _, sh := range bcp.Replsets {
-		rs, ok := fl[sh.Name]
+		name := mapRS(sh.Name)
+		rs, ok := fl[name]
 		if !ok {
-			nors = append(nors, sh.Name)
+			nors = append(nors, name)
+			continue
+		} else if mapRevRS(name) != sh.Name {
+			nors = append(nors, name)
 			continue
 		}
 
@@ -425,7 +515,7 @@ func (r *Restore) setShards(bcp *pbm.BackupMeta) error {
 	}
 
 	if r.nodeInfo.IsLeader() && len(nors) > 0 {
-		return errors.Errorf("extra/unknown replica set found in the backup: %s", strings.Join(nors, ","))
+		return errors.Errorf("extra/unknown replica set found in the backup: %s", strings.Join(nors, ", "))
 	}
 
 	return nil
@@ -434,9 +524,13 @@ func (r *Restore) setShards(bcp *pbm.BackupMeta) error {
 var ErrNoDataForShard = errors.New("no data for shard")
 
 func (r *Restore) snapshotObjects(bcp *pbm.BackupMeta) (dump, oplog string, err error) {
+	mapRS := pbm.MakeRSMapFunc(r.rsMap)
+
 	var ok bool
 	for _, v := range bcp.Replsets {
-		if v.Name == r.nodeInfo.SetName {
+		name := mapRS(v.Name)
+
+		if name == r.nodeInfo.SetName {
 			dump = v.DumpName
 			oplog = v.OplogName
 			ok = true
@@ -465,7 +559,7 @@ func (r *Restore) snapshotObjects(bcp *pbm.BackupMeta) (dump, oplog string, err
 
 func (r *Restore) checkSnapshot(bcp *pbm.BackupMeta) error {
 	if bcp.Status != pbm.StatusDone {
-		return errors.Errorf("backup wasn't successful: status: %s, error: %s", bcp.Status, bcp.Error)
+		return errors.Errorf("backup wasn't successful: status: %s, error: %s", bcp.Status, bcp.Error())
 	}
 
 	if !version.Compatible(version.DefaultInfo.Version, bcp.PBMVersion) {
@@ -489,16 +583,6 @@ func (r *Restore) toState(status pbm.Status, wait *time.Duration) error {
 }
 
 func (r *Restore) RunSnapshot(dump string, bcp *pbm.BackupMeta) (err error) {
-	err = r.checkSnapshot(bcp)
-	if err != nil {
-		return err
-	}
-
-	err = r.setShards(bcp)
-	if err != nil {
-		return err
-	}
-
 	sr, err := r.stg.SourceReader(dump)
 	if err != nil {
 		return errors.Wrapf(err, "get object %s for the storage", dump)
@@ -536,6 +620,129 @@ func (r *Restore) RunSnapshot(dump string, bcp *pbm.BackupMeta) (err error) {
 	return nil
 }
 
+func (r *Restore) updateRouterConfig(ctx context.Context) error {
+	if len(r.rsMap) == 0 || !r.nodeInfo.IsSharded() {
+		return nil
+	}
+
+	if r.nodeInfo.IsConfigSrv() {
+		r.log.Debug("updating router config")
+		if err := updateRouterTables(ctx, r.cn.Conn, r.rsMap); err != nil {
+			return err
+		}
+	}
+
+	res := r.cn.Conn.Database(pbm.DB).RunCommand(ctx, primitive.M{"flushRouterConfig": 1})
+	return errors.WithMessage(res.Err(), "flushRouterConfig")
+}
+
+func updateRouterTables(ctx context.Context, m *mongo.Client, rsMap map[string]string) error {
+	if err := updateDatabasesRouterTable(ctx, m, rsMap); err != nil {
+		return errors.WithMessage(err, "databases")
+	}
+
+	if err := updateChunksRouterTable(ctx, m, rsMap); err != nil {
+		return errors.WithMessage(err, "chunks")
+	}
+
+	return nil
+}
+
+func updateDatabasesRouterTable(ctx context.Context, m *mongo.Client, rsMap map[string]string) error {
+	coll := m.Database("config").Collection("databases")
+
+	oldNames := make(primitive.A, 0, len(rsMap))
+	for k := range rsMap {
+		oldNames = append(oldNames, k)
+	}
+
+	q := primitive.M{"primary": primitive.M{"$in": oldNames}}
+	cur, err := coll.Find(ctx, q)
+	if err != nil {
+		return errors.WithMessage(err, "query")
+	}
+
+	models := make([]mongo.WriteModel, 0)
+	for cur.Next(ctx) {
+		var doc struct {
+			ID      string `bson:"_id"`
+			Primary string `bson:"primary"`
+		}
+		if err := cur.Decode(&doc); err != nil {
+			return errors.WithMessage(err, "decode")
+		}
+
+		m := mongo.NewUpdateOneModel()
+		m.SetFilter(primitive.M{"_id": doc.ID})
+		m.SetUpdate(primitive.M{"$set": primitive.M{"primary": rsMap[doc.Primary]}})
+
+		models = append(models, m)
+	}
+	if err := cur.Err(); err != nil {
+		return errors.WithMessage(err, "cursor")
+	}
+	if len(models) == 0 {
+		return nil
+	}
+
+	_, err = coll.BulkWrite(ctx, models)
+	return errors.WithMessage(err, "bulk write")
+}
+
+func updateChunksRouterTable(ctx context.Context, m *mongo.Client, rsMap map[string]string) error {
+	coll := m.Database("config").Collection("chunks")
+
+	oldNames := make(primitive.A, 0, len(rsMap))
+	for k := range rsMap {
+		oldNames = append(oldNames, k)
+	}
+
+	q := primitive.M{"history.shard": primitive.M{"$in": oldNames}}
+	cur, err := coll.Find(ctx, q)
+	if err != nil {
+		return errors.WithMessage(err, "query")
+	}
+
+	models := make([]mongo.WriteModel, 0)
+	for cur.Next(ctx) {
+		var doc struct {
+			ID      primitive.ObjectID `bson:"_id"`
+			Shard   string             `bson:"shard"`
+			History []struct {
+				Shard string `bson:"shard"`
+			} `bson:"history"`
+		}
+		if err := cur.Decode(&doc); err != nil {
+			return errors.WithMessage(err, "decode")
+		}
+
+		updates := primitive.M{}
+		if n, ok := rsMap[doc.Shard]; ok {
+			updates["shard"] = n
+		}
+
+		for i, h := range doc.History {
+			if n, ok := rsMap[h.Shard]; ok {
+				updates[fmt.Sprintf("history.%d.shard", i)] = n
+			}
+		}
+
+		m := mongo.NewUpdateOneModel()
+		m.SetFilter(primitive.M{"_id": doc.ID})
+		m.SetUpdate(primitive.M{"$set": updates})
+		models = append(models, m)
+	}
+	if err := cur.Err(); err != nil {
+		return errors.WithMessage(err, "cursor")
+	}
+	if len(models) == 0 {
+		return nil
+	}
+
+	_, err = coll.BulkWrite(ctx, models)
+	return errors.WithMessage(err, "bulk write")
+}
+
 func (r *Restore) distTxnChecker(done <-chan struct{}, ctxn chan pbm.RestoreTxn, txnSyncErr chan<- error) {
 	defer r.log.Debug("exit distTxnChecker")
 
@@ -660,9 +867,9 @@ func (r *Restore) applyOplog(chunks []pbm.OplogChunk, start, end *primitive.Time
 		// PBM versions) won’t be compatible - during the restore, PBM will treat such
 		// files as Snappy (judging by its suffix) but in fact, they are s2 files
 		// and restore will fail with snappy: corrupt input. So we try S2 in such a case.
-		lts, err = r.replyChunk(chnk.FName, chnk.Compression)
+		lts, err = r.replayChunk(chnk.FName, chnk.Compression)
 		if err != nil && errors.Is(err, snappy.ErrCorrupt) {
-			lts, err = r.replyChunk(chnk.FName, pbm.CompressionTypeS2)
+			lts, err = r.replayChunk(chnk.FName, pbm.CompressionTypeS2)
 		}
 		if err != nil {
 			return errors.Wrapf(err, "replay chunk %v.%v", chnk.StartTS.T, chnk.EndTS.T)
@@ -861,7 +1068,7 @@ func (r *Restore) snapshot(input io.Reader) (err error) {
 	return nil
 }
 
-func (r *Restore) replyChunk(file string, c pbm.CompressionType) (lts primitive.Timestamp, err error) {
+func (r *Restore) replayChunk(file string, c pbm.CompressionType) (lts primitive.Timestamp, err error) {
 	or, err := r.stg.SourceReader(file)
 	if err != nil {
 		return lts, errors.Wrapf(err, "get object %s form the storage", file)
