@@ -418,7 +418,7 @@ func (r *PhysRestore) copyFiles() error {
 				src := filepath.Join(r.bcp.Name, r.nodeInfo.SetName, f.Name+r.bcp.Compression.Suffix())
 				dst := filepath.Join(r.dbpath, f.Name)
 
-				err := os.MkdirAll(filepath.Dir(dst), os.ModeDir|0700)
+				err := os.MkdirAll(filepath.Dir(dst), os.ModeDir|0o700)
 				if err != nil {
 					return errors.Wrapf(err, "create path %s", filepath.Dir(dst))
 				}
@@ -592,9 +592,17 @@ func (r *PhysRestore) resetRS() error {
 		return errors.Wrap(err, "drop config.cache.chunks.config.system.sessions")
 	}
 
-	err = c.Database("config").Collection("system.sessions").Drop(ctx)
+	const retry = 5
+	for i := 0; i < retry; i++ {
+		err = c.Database("config").Collection("system.sessions").Drop(ctx)
+		if err == nil || !strings.Contains(err.Error(), "(BackgroundOperationInProgressForNamespace)") {
+			break
+		}
+		r.log.Debug("drop config.system.sessions: BackgroundOperationInProgressForNamespace, retrying")
+		time.Sleep(time.Second * time.Duration(i+1))
+	}
 	if err != nil {
-		return errors.Wrap(err, "drop from config.system.sessions")
+		return errors.Wrap(err, "drop config.system.sessions")
 	}
 
 	_, err = c.Database("local").Collection("system.replset").DeleteMany(ctx, bson.D{})
@@ -762,7 +770,7 @@ func (r *PhysRestore) prepareBackup(backupName string) (err error) {
 	}
 
 	if r.bcp.Status != pbm.StatusDone {
-		return errors.Errorf("backup wasn't successful: status: %s, error: %s", r.bcp.Status, r.bcp.Error)
+		return errors.Errorf("backup wasn't successful: status: %s, error: %s", r.bcp.Status, r.bcp.Error())
 	}
 
 	mgoV, err := r.node.GetMongoVersion()
